#include "state.hpp"

State::State () {
	//
}

void State::error () {
	ERROR = true;
	image.clear();
}

bool State::imageExists (string& image) {
    struct stat buffer;   
    return (stat (image.c_str(), &buffer) == 0); 
}

void State::setAction (json& msgData) {
	action = static_cast<Action>(msgData["action"]);
	switch (action) {
		case NONE:
			cout << "Action = NONE" << endl;
			break;
		case LOAD:
			cout << "Action = LOAD" << endl;
			break;
		case DISPLAY:
			cout << "Action = DISPLAY" << endl;
			break;
		case STOP:
			cout << "Action = STOP" << endl;
			break;
	}
}

void State::setImage (json& msgData) {
	image = msgData["image"];
	if (!imageExists(image)) {
		error();
		cerr << "Image " << msgData["image"] << " does not exist" << endl;
		return;
	}
	cout << "Image = " << image << endl;
}

void State::setMode (json& msgData) {
	mode = static_cast<Mode>(msgData["mode"]);
	cout << "Mode = " << mode << endl;
}

void State::setPosition (json& msgData) {
	x = msgData["position"]["x"];
	y = msgData["position"]["y"];
	w = msgData["position"]["w"];
	h = msgData["position"]["h"];
	cout << "Position[x] = " << x << " [y] = " << y << " [w] = " << w << " [h] = " << h << endl;
}

void State::setExposure (json& msgData) {
	if (msgData["exposure"].size() == 0) {
		setNoExposure();
	} else if (msgData["exposure"].size() == 1) {
		exposure = { msgData["exposure"][0] };
		cout << "Exposure = " << msgData["exposure"][0] << " ms" << endl;
	} else if (msgData["exposure"].size() == 3) {
		exposure = { msgData["exposure"][0], msgData["exposure"][1], msgData["exposure"][2] };
		cout << "Exposure[0] = " << msgData["exposure"][0] << " ms, [1] = " << msgData["exposure"][1] << " ms, [2] = " << msgData["exposure"][2] << " ms" << endl;
	} else {
		error();
		cerr << "Exposure array is incorrect length, " << msgData["exposure"].size() << endl;
		return;
	}
}

void State::setNoExposure () {
	exposure = {};
	cout << "No exposure, display indefinitely" << endl;
}

void State::receiveMessage (string msgString) {
	ERROR = false;
	json msgData = json::parse(msgString);
	if (msgData.contains("action")) {
		setAction(msgData);
	} else {
		action = NONE;
		error();
		cerr << "Received invalid message: " << msgString << endl;
		return;
	}

	if (action == LOAD && msgData.contains("image")) {
		setImage(msgData);
	}

	if (action == LOAD && msgData.contains("mode")) {
		setMode(msgData);
	}

	if (action == LOAD && msgData.contains("position")) {
		setPosition(msgData);
	}

	if (action == DISPLAY && msgData.contains("exposure")) {
		setExposure(msgData);
	} else if (action == DISPLAY && !msgData.contains("exposure")) {
		setNoExposure();
	}

	setHasAction();
}

string State::createMessage (bool success) {
	json msgData = {
		{ "action", action },
		{ "success", success }
	};
	return msgData.dump();
}